"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ForemanLambda = void 0;
const common_utils_1 = require("@fluidframework/common-utils");
const protocol_definitions_1 = require("@fluidframework/protocol-definitions");
const core = __importStar(require("@fluidframework/server-services-core"));
const server_services_telemetry_1 = require("@fluidframework/server-services-telemetry");
// TODO: Move this to config.
const RequestWindowMS = 15000;
class ForemanLambda {
    constructor(messageSender, tenantManager, tokenGenerator, permissions, context, tenantId, documentId) {
        this.messageSender = messageSender;
        this.tenantManager = tenantManager;
        this.tokenGenerator = tokenGenerator;
        this.permissions = permissions;
        this.context = context;
        this.tenantId = tenantId;
        this.documentId = documentId;
        this.taskQueueMap = new Map();
        this.rateLimiter = new common_utils_1.RateLimiter(RequestWindowMS);
        // Make a map of every task and their intended queue.
        // eslint-disable-next-line guard-for-in, no-restricted-syntax
        for (const queueName in this.permissions) {
            for (const task of this.permissions[queueName]) {
                this.taskQueueMap.set(task, queueName);
            }
        }
    }
    close() { }
    async handler(message) {
        const boxcar = core.extractBoxcar(message);
        for (const baseMessage of boxcar.contents) {
            if (baseMessage.type === core.SequencedOperationType) {
                const sequencedMessage = baseMessage;
                // Only process "Help" messages.
                if (sequencedMessage.operation.type === protocol_definitions_1.MessageType.RemoteHelp) {
                    const helpMessage = JSON.parse(sequencedMessage.operation.data);
                    // Back-compat to play well with older client.
                    const helpContent = helpMessage.version
                        ? helpMessage.tasks.map((task) => `chain-${task}`)
                        : helpMessage.tasks;
                    await this.trackDocument(sequencedMessage.operation.clientId, sequencedMessage.tenantId, sequencedMessage.documentId, helpContent);
                }
            }
        }
        this.context.checkpoint(message);
    }
    // Sends help message for a task. Uses a rate limiter to limit request per clientId.
    async trackDocument(clientId, tenantId, docId, helpTasks) {
        var _a;
        const key = await this.tenantManager.getKey(tenantId);
        const queueTasks = this.generateQueueTasks(helpTasks);
        for (const queueTask of queueTasks) {
            const queueName = queueTask[0];
            const tasks = this.rateLimiter.filter(clientId, queueTask[1]);
            if (tasks.length > 0) {
                const scopes = [protocol_definitions_1.ScopeType.DocRead, protocol_definitions_1.ScopeType.DocWrite, protocol_definitions_1.ScopeType.SummaryWrite];
                const queueMessage = {
                    documentId: docId,
                    message: {
                        tasks,
                    },
                    tenantId,
                    token: this.tokenGenerator(tenantId, docId, key, scopes),
                };
                this.messageSender.sendTask(queueName, {
                    content: queueMessage,
                    type: "tasks:start",
                });
                (_a = this.context.log) === null || _a === void 0 ? void 0 : _a.info(`Request to ${queueName}: ${clientId}:${JSON.stringify(tasks)}`, {
                    messageMetaData: {
                        documentId: docId,
                        tenantId,
                    },
                });
                server_services_telemetry_1.Lumberjack.info(`Request to ${queueName}: ${clientId}:${JSON.stringify(tasks)}`, {
                    [server_services_telemetry_1.BaseTelemetryProperties.tenantId]: tenantId,
                    [server_services_telemetry_1.BaseTelemetryProperties.documentId]: docId,
                });
            }
        }
    }
    // From a list of task requests, figure out the queue for the tasks and return a map of <queue, takss[]>
    generateQueueTasks(tasks) {
        // Figure out the queue for each task and populate the map.
        const queueTaskMap = new Map();
        for (const task of tasks) {
            const queue = this.taskQueueMap.get(task);
            if (queue) {
                let queueTasks = queueTaskMap.get(queue);
                if (!queueTasks) {
                    queueTasks = [];
                    queueTaskMap.set(queue, queueTasks);
                }
                queueTasks.push(task);
            }
        }
        return queueTaskMap;
    }
}
exports.ForemanLambda = ForemanLambda;
//# sourceMappingURL=lambda.js.map